#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file set_2dcdp_pram_gui.py
 @brief 2DCDP system parameter set GUI component
 @date $Date$

 @author 株式会社 東日本計算センター

"""
import sys
import time
sys.path.append(".")

import tkMessageBox
import copy
import datetime

# Import RTM module
import RTC
import OpenRTM_aist

import RTC_2DCDPSystemControl_idl
import ImageLoaderControl_idl
import create_gui_class
# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
import RTC_2DCDPSystemContServ
import RTC_2DCDPSystemContServ__POA
import ImageLoaderControlServ
import ImageLoaderControlServ__POA
import ImageLoaderSystemContServ
import ImageLoaderSystemContServ__POA

set_gui = create_gui_class.CreateGUI()

# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
set_2dcdp_pram_gui_spec = [
    "implementation_id", "set_2dcdp_pram_gui",
    "type_name",         "set_2dcdp_pram_gui",
    "description",       "2DCDP system parameter set GUI component",
    "version",           "1.0.0",
    "vendor",            "EAC",
    "category",          "GUI",
    "activity_type",     "STATIC",
    "max_instance",      "1",
    "language",          "Python",
    "lang_type",         "SCRIPT",
    ""]
# </rtc-template>

##
# @class set_2dcdp_pram_gui
# @brief 2DCDP system parameter set GUI component
#
# [機能仕様書_２DCDPパラメータ設定RTC]参照
#
# [機能仕様書_２DCDPパラメータ設定RTC]参照
#
# [機能仕様書_２DCDPパラメータ設定RTC]参照
#
# http://docs.python.jp/2/library/tkinter.html
#
#


class set_2dcdp_pram_gui(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_outParameter = RTC.TimedStringSeq(RTC.Time(0, 0), [])
        """
        ユーザー操作により設定された２DCDPの Parameterを2DCDP
        RTCに通知するためのTimedStringSeq型の配列。
         - Type: RTC::TimedStringSeq
         - Number: 13
         - Semantics: [フィールド:tm]
                      ‘datetime’モジュールを使用して時間情報を取得し設定する。
                      sec:日付情報(日付/時刻/分/秒)を秒に変換し設定する。
                      例:x月10日15時30分45秒
                      10*24*3600+15*3600+30*60+45=E0925H
                      nsec:ナノ秒を設定
                      [フィールド:data]
                      コマンド名とParameterを設定する。
                      0：コマンド名
                      “PARAMETER_SET”：パラメータセットボタンが押されたことを通知す
                      る。
                      “DEFALUT_PARAMETER_SET”：デフォルトパラメータセットボタンが押
                      されたことを通知する。
                      1:タグ名
                      ユーサーがGUIのテキストBoxに入力した文字(半角英数字20文字以内)
                      を設定する。
                      2：取り出し数指定
                      ユーザーがラジオボタンで指定した値(“1”か”2”)を設定する。
                      3:上り連続縮小制限指定
                      ユーザーがラジオボタンで指定した値(“1”か”2”)を設定する。
                      4:下り連続縮小制限指定
                      ユーザーがラジオボタンで指定した値(“1”か”2”)を設定する。
                      5-7:背景色(出力)指定
                      ユーザーが指定したred/green/blueそれぞれの色情報の値(“0”-“2
                      55”)を設定する。
                      8-10:背景色(参照)指定
                      ユーザーが指定したred/green/blueそれぞれの色情報の値(“0”-“2
                      55”)を設定する。
         - Unit: -
         - Frequency: 250
         - Operation Cycle: 250
        """
        self._outParameterOut = OpenRTM_aist.OutPort(
            "outParameter", self._d_outParameter)

        """
        2DCDP RTCのシステム制御
        Interface: リセット制御のI/Fを実行する。
        """
        self._RTC_2DCDPSystemContServPort = OpenRTM_aist.CorbaPort(
            "RTC_2DCDPSystemContServ")
        """
        ImageLoader RTCのシステム制御
        Interface: リセット制御のI/Fを実行する。
        """
        self._ImageLoaderSystemContServPort = OpenRTM_aist.CorbaPort(
            "ImageLoaderSystemContServ")

        """
        2DCDP System Control RTCのリセット制御
         - Argument:      Resetのトリガ
                          　RESET_TRIG_ON：Reset On
                          　RESET_TRIG_OFF：Reset Off
         - Return Value:  -
         - Exception:     -
         - PreCondition:  -
         - PostCondition: -
        """
        self._RTC_2DCDPSystemContReset = OpenRTM_aist.CorbaConsumer(
            interfaceType=RTC_2DCDPSystemContServ.RTC_2DCDPSystemContReset)
        """
        ImageLoaderRTCのリセット制御
         - Argument:      Resetのトリガ
                          RESET_TRIG_ON：Reset On
                        　RESET_TRIG_OFF：Reset Off
         - Return Value:  -
         - Exception:     -
         - PreCondition:  -
         - PostCondition: -
        """
        self._ImageLoaderReset = OpenRTM_aist.CorbaConsumer(
            interfaceType=ImageLoaderSystemContServ.ImageLoaderReset)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">

        # </rtc-template>

    ##
    # Initialization of RTC
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    # @pre nothing special
    # @post nothing special
    #
    def onInitialize(self):
        # Bind variables and configuration variable

        # Set InPort buffers

        # Set OutPort buffers
        self.addOutPort("outParameter", self._outParameterOut)

        # Set service provider to Ports

        # Set service consumers to Ports
        self._RTC_2DCDPSystemContServPort.registerConsumer(
            "RTC_2DCDPSystemContReset",
            "RTC_2DCDPSystemContServ::RTC_2DCDPSystemContReset",
            self._RTC_2DCDPSystemContReset)
        self._ImageLoaderSystemContServPort.registerConsumer(
            "ImageLoaderReset",
            "ImageLoaderSystemContServ::ImageLoaderReset",
            self._ImageLoaderReset)

        # Set CORBA Service Ports
        self.addPort(self._RTC_2DCDPSystemContServPort)
        self.addPort(self._ImageLoaderSystemContServPort)

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # Activate the RTC
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre nothing special
        # @post nothing special
        #
    def onActivated(self, ec_id):
        self._d_outParameter.data = []
        self._d_outParameter.tm.sec = 0
        self._d_outParameter.tm.nsec = 0
        self.out = []
        self.flg = False

        return RTC.RTC_OK

        ##
        # Deactivate the RTC
        #
        # The deactivated action (Active state exit action)
        # former rtc_active_exit()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre nothing special
        # @post nothing special
        #
    def onDeactivated(self, ec_id):

        return RTC.RTC_OK

        ##
        # Periodic task of RTC. This task send parameter to 2DCDP RTC.
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre nothing special
        # @post nothing special
        #
    def onExecute(self, ec_id):
        if self.flg:
            print "onExecute write"

            timenow = datetime.datetime.now()

            daysec = (timenow.day * 24 * 3600)
            hoursec = (timenow.hour * 3600)
            minsec = (timenow.minute * 60)

            self._d_outParameter.tm.sec = (daysec + hoursec +
                                           minsec + timenow.second)
            self._d_outParameter.tm.nsec = (timenow.microsecond * 1000)

            # 書き込みデータの設定
            del self._d_outParameter.data[:]
            self._d_outParameter.data = copy.deepcopy(self.out)

            # dataポートへの書き込み実行
            self.ret = self._outParameterOut.write()
            if not self.ret:
                print "!write error"

            self.flg = False

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The aborting action when main logic error occurred.
    #    # former rtc_aborting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onAborting(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The reset action that is invoked resetting
    #    # This is same but different the former rtc_init_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def set_out_write(self, value):
        print "set_out_write()"

        self.flg = False

        if value[0] is "PARAMETER_SET" or value[0] is "DEFAULT_PARAMETER_SET":
            if isinstance(value[1], str) and value[1].isalnum() is True:
                del self.out[:]
                self.out.append(value[0])
                self.out.append(value[1])

                for i in range(2, 11):
                    self.out.append(str(value[i]))

                self.flg = True

            else:
                print u"input error"
                tkMessageBox.showerror(
                    u"input error",
                    u"Output file identification tag designation "
                    u"should be entered in alphanumeric characters."
                    u"\nAfter the input, press the [PARAMETER SET] "
                    u"button again.")
        else:
            pass

    def set_out_reset(self):
        print "set_out_reset()"
        # ResetのServicePort実行
        self._ImageLoaderReset._ptr().ReqImageLoaderReset(1)
        self._RTC_2DCDPSystemContReset._ptr().Req2DCDPSystemContReset(1)


def set_2dcdp_pram_guiInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=set_2dcdp_pram_gui_spec)
    manager.registerFactory(profile,
                            set_2dcdp_pram_gui,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    set_2dcdp_pram_guiInit(manager)

    # Create a component
    comp = manager.createComponent("set_2dcdp_pram_gui")
    set_gui.master.title("Set2DCDPparameterGUI")
    set_gui.set_on_write(comp.set_out_write)
    set_gui.set_on_reset(comp.set_out_reset)


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager(True)
    set_gui.mainloop()

if __name__ == "__main__":
    main()
